package ru.kgsu.it;

import java.io.IOException;
import java.sql.*;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import static ru.kgsu.it.Utils.*;

public class Main {

    /**
     * Функция для получения соединения с базой
     *
     * @return Объект соединения с базой
     */
    private static Connection getDBConnection() throws SQLException {
        Properties properties = new Properties();
        properties.setProperty(USER_KEY_PROPERTY, USER);
        properties.setProperty(PASSWORD_KEY_PROPERTY, PASSWORD);

        return DriverManager.getConnection(URL_CONNECTION_DATABASE, properties);
    }

    /**
     * Функция для добавления пользователя в базу
     *
     * @param conn Соединение с базой
     */
    private static void addUser(Connection conn) throws IOException, SQLException {
        printSeparator(true, false);

        System.out.println("Добавление нового пользователя");

        String login = readString("Введите логин пользователя: ");

        if (checkUser(conn, login)) {
            System.out.println("Ошибка: Данный пользователь уже существует");
            printSeparator(false, true);
            return;
        }

        String name = readString("Введите имя пользователя: ");
        String surname = readString("Введите фамилию пользователя: ");

        PreparedStatement insertStatement = conn.prepareStatement(INSERT_USER_SQL);
        insertStatement.setString(1, login);
        insertStatement.setString(2, name);
        insertStatement.setString(3, surname);
        insertStatement.execute();

        System.out.println("Операция успешно выполнена");
        printSeparator(false, true);
    }


    /**
     * Функция для удаления пользователя из базы
     *
     * @param conn Соединение с базой
     */
    private static void removeUser(Connection conn) throws IOException, SQLException {
        printSeparator(true, false);

        System.out.println("Удаление пользователя");

        String login = readString("Введите логин пользователя: ");

        if (!checkUser(conn, login)) {
            System.out.println("Ошибка: Данный пользователь не существует");
            printSeparator(false, true);
            return;
        }

        PreparedStatement removeStatement = conn.prepareStatement(DELETE_USER_SQL);
        removeStatement.setString(1, login);
        removeStatement.executeUpdate();

        System.out.println("Операция успешно выполнена");
        printSeparator(false, true);
    }


    /**
     * Функция для измения информации о пользователе в базе
     *
     * @param conn Соединение с базой
     */
    private static void updateUser(Connection conn) throws IOException, SQLException {
        printSeparator(true, false);

        System.out.println("Редактирование пользователя");

        String login = readString("Введите логин пользователя: ");

        if (!checkUser(conn, login)) {
            System.out.println("Ошибка: Данный пользователь не существует");
            printSeparator(false, true);
            return;
        }

        String name = readString("Введите новое имя пользователя: ");
        String surname = readString("Введите новую фамилию пользователя: ");

        PreparedStatement updateStatement = conn.prepareStatement(UPDATE_USER_SQL);
        updateStatement.setString(1, name);
        updateStatement.setString(2, surname);
        updateStatement.setString(3, login);
        updateStatement.execute();

        System.out.println("Операция успешно выполнена");
        printSeparator(false, true);
    }


    /**
     * Функция для вывода всех пользователей из базы
     *
     * @param conn Соединение с базой
     */
    private static void printUsers(Connection conn) throws SQLException {
        List<User> users = getUsersFromDb(conn);

        printSeparator(true, false);

        if (users.size() == 0) {
            System.out.println("Список пользователей пуст");
            printSeparator(false, true);
            return;
        }

        System.out.println("Список пользователей: ");
        users.forEach(System.out::println);

        printSeparator(false, true);
    }


    /**
     * Функция для получения всех пользователей из базы
     *
     * @param conn Соединение с базой
     * @return Список пользователей
     */
    private static List<User> getUsersFromDb(Connection conn) throws SQLException {
        List<User> users = new ArrayList<>();

        Statement statement = conn.createStatement();

        ResultSet rs = statement.executeQuery(SELECT_USER_SQL);
        while (rs.next()) {
            String login = rs.getString("login");
            String name = rs.getString("name");
            String surname = rs.getString("surname");
            User user = new User(name, surname, login);
            users.add(user);
        }

        return users;
    }

    public static void main(String[] args) throws SQLException, IOException {
        // Получаем соединение с базой
        Connection conn = getDBConnection();

        while (true) {
            // Пока мы не нажали выход из программы

            // Выводим меню
            printMenu();

            // Просим ввести номер пункта меню
            int number = 0;
            while (number < 1 || number > 5) number = readNumber("> ");

            // Выполняем команду соответствующую введенному номеру пункту меню
            if (number == ADD_USER) addUser(conn);
            else if (number == REMOVE_USER) removeUser(conn);
            else if (number == UPDATE_USER) updateUser(conn);
            else if (number == PRINT_USERS) printUsers(conn);
            else break;
        }

        // Закрываем соединение с базой
        conn.close();
    }
}
